#!/usr/bin/perl

#defaults
$verbose = 0;	#print extra information
$assemble = 0; 	#mode, decompose string if 0 
$decompose = 0;
$interactive_mode = 0;	#input file specified
$make_script = 0; #if 1, make a script for sim-crowbar

sub print_help_info
{
    #print helpful information
    print STDOUT "\n";
    print STDOUT "\n";
    print STDOUT "Script for assembling/decomposing Crowbar configuration strings\n";
    print STDOUT "\tJason Cantin, 8/2000\n";
    print STDOUT "\tUsage: %assemble_crowbar [-v(a|s|d)] [<input_filename>]\n";
    print STDOUT "\n";
    print STDOUT "\tUse -v for verbose operation\n";
    print STDOUT "\tUse -a to assemble a string (only) from a description\n";
    print STDOUT "\tUse -s to assemble description into a simulator script\n";
    print STDOUT "\tUse -d to decompose a string into a description\n";
    print STDOUT "\tUse -h for help\n";
    print STDOUT "\n";
    print STDOUT "\tLOGIC_BLOCK <0-7>\n";
    print STDOUT "\t\tINPUT0: <lut_input>\n";
    print STDOUT "\t\tINPUT1: <lut_input>\n";
    print STDOUT "\t\tINPUT2: <lut_input>\n";
    print STDOUT "\t\tMINTERMS: [<minterm>{,<minterm>}]\n";
    print STDOUT "\n";
    print STDOUT "INPUT0 and INPUT2 can be any of { N, E, W, S }\n";
    print STDOUT "INPUT1 can be any of { N, S, L, F }\n";
    print STDOUT " where L is the lookup-table output and F is the flop\n";
    print STDOUT " There can be 0-8 minterms, where 7 represents INPUT0*INPUT1*INPUT2,\n";
    print STDOUT " 6 represents INPUT0'*INPUT1*INPUT2,\n";
    print STDOUT "	and so on.\n";
    print STDOUT "\n";
    print STDOUT "\tSENSOR_BLOCK <0-3>\n";
    print STDOUT "\t\tTHRESHOLD: <threshold_of_receiver>\n";
    print STDOUT "\t\tOUTPUT: <mux_output>\n";
    print STDOUT "\n";
    print STDOUT "THRESHOLD can be any of { UHS, VHS, MHS, HS, LS, MLS, VLS, ULS }\n";
    print STDOUT "Ranges from Ultra-high sensitivity to Ultra-low sensitivity \n";
    print STDOUT "OUTPUT can be any of { N, E, W, S, R, ClkA, ClkB}\n";
    print STDOUT " where R is the receiver\n";
    print STDOUT "\n";
    print STDOUT "Configuration string is binary, with a spaces between fields\n";
    print STDOUT "Load first (leftmost) bit onto device first\n";
    print STDOUT "\n";
} #print_help_info

if ($#ARGV == -1)
{
    #no arguments, prompt for input file name
    $interactive_mode = 1;
    &print_help_info;
}
elsif($#ARGV == 0)
{
 #filename --use default options
    $assemble = 1;
    $verbose = 0;
    $decompose = 0;
    $make_script = 0;
    $input_file = $ARGV[0];
}
else
{
    #one or more arguments
    if ($#ARGV > 0)
    {
	#one argument, is it a filename or options? --look for dash
	if ($ARGV[0] =~ /^-(.*)/)
	{
	    $options = $1;

	    #parse string
	    $verbose = (index($options, "v") != -1);   
	    if (index($options, "d") != -1)
	    {
		$assemble = 0;
                $decompose = 1;
	    }
	    if (index($options, "a") != -1)
	    {
		if($decompose == 1)
		{
		    &print_help_info;
		    print STDERR "Error, conflicting flags: a,d!\n";
		    die;
		}
		$assemble = 1;
                $decompose = 0;
	    }
	    if ($#ARGV == 1)
	    {
		#two, next one is input file
		$input_file = $ARGV[1];
	    }
	    else
	    {
		#one argument, it was options
		$interactive_mode = 1;
	    }
	    if (index($options, "h") != -1)
	    {
		#just looking for help;
		&print_help_info;
		exit 0;
	    } #if

	    if (index($options, "s") != -1)
	    {
		#create a CROWBAR-simulator script;
		$make_script = 1;
		if(($assemble == 1)||($decompose == 1))
		{
		    &print_help_info;
		    print STDERR "Error, conflicting flags: s and a or d!\n";
		    die;
		}
	    } #if

	} #if
    } #if
} #else

if($interactive_mode)
{
    #prompt for input file name
    if($assemble)
    {
	print STDOUT "Enter filename of logical description\n";
    } 
    else
    {
	print STDOUT "Enter filename of string to decompose\n";
    }
    sysread STDIN, $input_file, 100;
    if($verbose)
    {
	print STDOUT "Input file name: ".$input_file."\n";
    }
} #if

#open input file
if($assemble or $make_script)
{
 open INPUT, "<$input_file.description" or die "Can't open $input_file.description: $!\n";
}
else
{
 open INPUT, "<$input_file.config" or die "Can't open $input_file.config: $!\n";
}

if($assemble)
{
 open OUTPUT, ">$input_file.config" or die "Can't open $input_file.config: $!\n";
}
elsif($make_script)
{
 open OUTPUT, ">$input_file.sim-crowbar" or die "Can't open $input_file.sim-crowbar: $!\n";
}
else
{
 open OUTPUT, ">$input_file.description" or die "Can't open $input_file.description: $!\n";
}

if($assemble or $make_script)
{
    #initialize configuration to defaults
    $logic_block_declaration = 0;
    $sensor_block_declaration = 0;
    for($i = 0; $i < 8; $i++)
    {
	@mux0_select[$i]="00";
	@mux1_select[$i]="00";
	@mux2_select[$i]="00";
	@lut[$i]="00000000";
    }
    for($i = 0; $i < 4; $i++)
    {
	@threshold[$i]=000;
	@output_select[$i]=000;
    }

    print STDOUT "Initializing...\n" if $verbose;

    #process input description, output config string
    while(<INPUT>)
    {
	#for each line in file
	if(/^#(.*)/)
	   {
	       #comment, do nothing unless verbose
	       print STDOUT $_ if $verbose;
	   } #if
	   else
	   {
	       #We've got something, deploy the regular expressions!
	       #For logic blocks:
	       if(/^\s*logic_block\s*(\d)\s*/i)
	       {
		   #We've got a logic block declaration, set block_id
		   $logic_block_id = $1;
		   print "Processing Logic Block ".$1."\n" if $verbose;
		   $logic_block_declaration = 1;
		   $sensor_block_declaration = 0;
		   if($logic_block_id > 7)
		   {
		       die "Error: Logic block ID must be between 0 and 7 for Crowbar\n";
		   }
	       } #if

	       #For input declarations:
	       if(/^\s*input\s*(\d)\s*:\s*(\w)/i)
	       {
		   if($logic_block_declaration == 0)
		   {
		       die "Error: Input declaration outside of Logic Block declaration\n";
		   }
		   if($1 > 2)
		   {
		       die "Error: Only 3 inputs (0-2) per Logic block\n";
		   }
		   elsif($1 == 1)
		   {
		       if(lc($2) eq "s")
		       {
			   $select = "00";
		       }
		       elsif(lc($2) eq "f")
		       {
			   $select = "01";
		       }
		       elsif(lc($2) eq "n")
		       {
			   $select = "10";
		       }
		       elsif(lc($2) eq "l")
		       {
			   $select = "11";
		       }
		       else
		       {
			   die "Error: Unsupported input type for input1\n";
		       }
		   } #mux1
		   elsif(($1 == 0) or ($1 == 2))
		   {
		       if(lc($2) eq "s")
		       {
			   $select = "00";
		       }
		       elsif(lc($2) eq "w")
		       {
			   $select = "01";
		       }
		       elsif(lc($2) eq "n")
		       {
			   $select = "10";
		       }
		       elsif(lc($2) eq "e")
		       {
			   $select = "11";
		       }
		       else
		       {
			   die "Error: Unsupported input type for input0,2\n";
		       }
		   } #mux0,2
		   @{mux.$1._select}[$logic_block_id] = $select;
	       } #if

	       #For lut declarations
	       if(/^\s*minterms\s*:\s*((\w\s*,\s*)*(\w))/i)
	       {
		   if($logic_block_declaration == 0)
		   {
		       die "Error: Minterm declaration outside of Logic Block declaration\n";
		   }
		   @raw_input = split / /, $1;
		   $minterms = join "", @raw_input;
		   @minterms = split /,/, $minterms;
		   sort numerically @minterms;
		   @lut_config = (0, 0, 0, 0, 0, 0, 0, 0);
		   for($j = 0; $j <=$#minterms; $j++)
		   {
		       @lut_config[@minterms[$j]] = 1;
		   }
		   @lut[$logic_block_id] = join "",@lut_config;
		   #print @lut[$logic_block_id]."\n";
	       } #if

	       #For sensor blocks
	       if(/^\s*sensor_block\s*(\d)\s*/i)
	       {
		   #We've got a sensor block declaration, set block_id
		   $sensor_block_id = $1;
		   $logic_block_declaration = 0;
		   $sensor_block_declaration = 1;
		   print "Processing Sensor Block ".$1."\n" if $verbose;
		   if($sensor_block_id > 3)
		   {
		       die "Error: Sensor block ID must be between 0 and 3 for Crowbar\n";
		   }
	       } #if

	       if(/^\s*threshold\s*:\s*(\w+)/i)
	       {
		   if($sensor_block_declaration == 0)
		   {
		       die "Error: Threshold declaration outside of Sensor Block declaration\n";
		   }
		   if(lc($1) eq "uhs")
		   {
		       $threshold = "000";
		   }
		   elsif(lc($1) eq "vhs")
		   {
		       $threshold = "100";
		   }
		   elsif(lc($1) eq "mhs")
		   {
		       $threshold = "010";
		   }
		   elsif(lc($1) eq "hs")
		   {
		       $threshold = "110";
		   }
		   elsif(lc($1) eq "ls")
		   {
		       $threshold = "001";
		   }
		   elsif(lc($1) eq "mls")
		   {
		       $threshold = "101";
		   }
		   elsif(lc($1) eq "vls")
		   {
		       $threshold = "011";
		   }
		   elsif(lc($1) eq "uls")
		   {
		       $threshold = "111";
		   }
		   else
		   {
		       die "Error: Unsupported threshold for receiver\n";
		   }
		   @threshold[$sensor_block_id] = $threshold;
	       } #if

	       if(/^\s*output\s*:\s*(\w+)/i)
	       {
		   if($sensor_block_declaration == 0)
		   {
		       die "Error: sensor output declaration outside of Sensor Block declaration\n";
		   }
		   if(lc($1) eq "n")
		   {
		       $output = "010";
		   }
		   elsif(lc($1) eq "e")
		   {
		       $output = "110";
		   }
		   elsif(lc($1) eq "w")
		   {
		       $output = "100";
		   }
		   elsif(lc($1) eq "s")
		   {
		       $output = "000";
		   }
		   elsif(lc($1) eq "clka")
		   {
		       $output = "101";
		   }
		   elsif(lc($1) eq "clkb")
		   {
		       $output = "011";
		   }
		   else
		   {
		       die "Error: Unsupported output for Sensor Block\n";
		   }
		   @output[$sensor_block_id] = $output;
	       } #if
	   } #else, not a comment
       } #while
	#done reading input, create string
	print "Creating binary string...\n" if $verbose;
	$config_string = @mux1_select[0]." ".@mux2_select[0]." ".@mux0_select[0];
	$config_string = $config_string." ".@mux1_select[1]." ".@mux2_select[1]." ".@mux0_select[1];
	$config_string = $config_string." ".@threshold[1]." ".@output[1]." ".@lut[1];
	$config_string = $config_string." ".@lut[0]." ".@threshold[0]." ".@output[0];
	$config_string = $config_string." ".@mux1_select[2]." ".@mux2_select[2]." ".@mux0_select[2];
	$config_string = $config_string." ".@mux1_select[3]." ".@mux2_select[3]." ".@mux0_select[3];
	$config_string = $config_string." ".@mux1_select[4]." ".@mux2_select[4]." ".@mux0_select[4];
	$config_string = $config_string." ".@mux1_select[5]." ".@mux2_select[5]." ".@mux0_select[5];
	$config_string = $config_string." ".@lut[5]." ".@lut[4]." ".@lut[3]." ".@lut[2];
	$config_string = $config_string." ".@mux1_select[6]." ".@mux2_select[6]." ".@mux0_select[6];
	$config_string = $config_string." ".@mux1_select[7]." ".@mux2_select[7]." ".@mux0_select[7];
	$config_string = $config_string." ".@threshold[3]." ".@output[3]." ".@lut[7]." ".@lut[6];
	$config_string = $config_string." ".@threshold[2]." ".@output[2];

        if($make_script)
	{
	    print OUTPUT "|This is a script for sim-crowbar, the prototype simulator\n";
	    print OUTPUT "|It supplies power to the chip, loads the configuration,\n";
            print OUTPUT "| resets the chip, and sets all inputs low.\n";
            print OUTPUT "|You can then apply signals and view outputs!\n";
	    print OUTPUT "\n";
            print OUTPUT "|Power-up:\n";
            print OUTPUT "h VDD Pad_VDD1 Pad_Vdd2\n";
            print OUTPUT "l GND Pad_GND1 Pad_GND2\n";
            print OUTPUT "|Next step configures:\n";
            print OUTPUT "c ";
	} #stuff before config string in script

	print OUTPUT $config_string."\n";

        if($make_script)
	{
            print OUTPUT "|Configuration complete\n";
            print OUTPUT "l CLK_B\n";
            print OUTPUT "|Reset chip\n";
            print OUTPUT "l RST_L |asserted low\n";
            print OUTPUT "s 1\n";
            print OUTPUT "h RST_L\n";
	    print OUTPUT "s 1\n";
            print OUTPUT "| Initialize pins to low\n";
            print OUTPUT "l L5NIN_H S1NIN_H S1WIN_H SCANIN_H\n";
            print OUTPUT "l L1WIN_H L0WIN_H S0WIN_H S0SIN_H\n";
            print OUTPUT "l L2SIN_H S2SIN_H TEST_H S2EIN_H\n";
            print OUTPUT "l L6EIN_H L7EIN_H S3EIN_H S3NIN_H\n";
            print OUTPUT "s 100\n";
            print OUTPUT "| Chip configured, reset, and initialized.\n";
            print OUTPUT "| Now let's do some real damage...\n\n";
	} #stuff after config string in script

    }
    else
{
    #decompose a string, output a logical description
    $config_string = "";
    while(<INPUT>)
    {
	#for each line in file
	if(/^#(.*)/)
	   {
	       #comment, do nothing unless verbose
	       print STDOUT $_ if $verbose;
	   } #if
	   else
	   {
	       $config_string = $config_string.$_;  
	   }
       } #while
	#strip out whitespace
	@config_array = split /\s+/, $config_string;
	$config_string = join '', @config_array;
	@config_array = split //, $config_string;
	if($#config_array != 135)
	{
	    die "String length is not correct --136 bits\n";
	}
	print "Parsing input string...\n" if $verbose;
	#chop it up
	@mux1_select[0] = join '', splice(@config_array, 0, 2);
	@mux2_select[0] = join '', splice(@config_array, 0, 2);
	@mux0_select[0] = join '', splice(@config_array, 0, 2);
	@mux1_select[1] = join '', splice(@config_array, 0, 2);
	@mux2_select[1] = join '', splice(@config_array, 0, 2);
	@mux0_select[1] = join '', splice(@config_array, 0, 2);
	@threshold[1] =  join '', splice(@config_array, 0, 3);
	@output[1] =  join '', splice(@config_array, 0, 3);
	@lut[1] =  join '', splice(@config_array, 0, 8);
	@lut[0] =  join '', splice(@config_array, 0, 8);
	@threshold[0] =  join '', splice(@config_array, 0, 3);
	@output[0] =  join '', splice(@config_array, 0, 3);
	@mux1_select[2] = join '', splice(@config_array, 0, 2);
	@mux2_select[2] = join '', splice(@config_array, 0, 2);
	@mux0_select[2] = join '', splice(@config_array, 0, 2);
	@mux1_select[3] = join '', splice(@config_array, 0, 2);
	@mux2_select[3] = join '', splice(@config_array, 0, 2);
	@mux0_select[3] = join '', splice(@config_array, 0, 2);
	@mux1_select[4] = join '', splice(@config_array, 0, 2);
	@mux2_select[4] = join '', splice(@config_array, 0, 2);
	@mux0_select[4] = join '', splice(@config_array, 0, 2);
	@mux1_select[5] = join '', splice(@config_array, 0, 2);
	@mux2_select[5] = join '', splice(@config_array, 0, 2);
	@mux0_select[5] = join '', splice(@config_array, 0, 2);
	@lut[5] =  join '', splice(@config_array, 0, 8);
	@lut[4] =  join '', splice(@config_array, 0, 8);
	@lut[3] =  join '', splice(@config_array, 0, 8);
	@lut[2] =  join '', splice(@config_array, 0, 8);
	@mux1_select[6] = join '', splice(@config_array, 0, 2);
	@mux2_select[6] = join '', splice(@config_array, 0, 2);
	@mux0_select[6] = join '', splice(@config_array, 0, 2);
	@mux1_select[7] = join '', splice(@config_array, 0, 2);
	@mux2_select[7] = join '', splice(@config_array, 0, 2);
	@mux0_select[7] = join '', splice(@config_array, 0, 2);
	@threshold[3] =  join '', splice(@config_array, 0, 3);
	@output[3] =  join '', splice(@config_array, 0, 3);
	@lut[7] =  join '', splice(@config_array, 0, 8);
	@lut[6] =  join '', splice(@config_array, 0, 8); 
	@threshold[2] =  join '', splice(@config_array, 0, 3);
	@output[2] =  join '', splice(@config_array, 0, 3);

	print STDOUT "Printing logical description...\n" if $verbose;
	print STDOUT "\n";
	#print out in meaningful format
	for($j = 0; $j < 8; $j++)
	{
	    print OUTPUT "LOGIC_BLOCK $j\n";
	    print OUTPUT " INPUT0: ";
	    print OUTPUT "S\n" if $mux0_select[$j] eq "00";
	    print OUTPUT "W\n" if $mux0_select[$j] eq "01";
	    print OUTPUT "N\n" if $mux0_select[$j] eq "10";
	    print OUTPUT "E\n" if $mux0_select[$j] eq "11";
	    print OUTPUT " INPUT1: ";
	    print OUTPUT "S\n" if $mux1_select[$j] eq "00";
	    print OUTPUT "F\n" if $mux1_select[$j] eq "01";
	    print OUTPUT "N\n" if $mux1_select[$j] eq "10";
	    print OUTPUT "L\n" if $mux1_select[$j] eq "11";
	    print OUTPUT " INPUT2: ";
	    print OUTPUT "S\n" if $mux2_select[$j] eq "00";
	    print OUTPUT "W\n" if $mux2_select[$j] eq "01";
	    print OUTPUT "N\n" if $mux2_select[$j] eq "10";
	    print OUTPUT "E\n" if $mux2_select[$j] eq "11";
	    print OUTPUT " MINTERMS: ";
	    @minterm_list = split //, @lut[$j];
	    $add_comma = 0;
	    for($k = 0; $k < 8; $k++)
	    {
		if(@minterm_list[$k])
		{
		    print OUTPUT ", " if $add_comma;
		    print OUTPUT ($k);
		    $add_comma=1;
		}
	    } #for
	    print OUTPUT "\n";
	    print OUTPUT "\n";
	}#for

	    for($j = 0; $j < 4; $j++)
	    {
		print OUTPUT "SENSOR_BLOCK $j\n";
		print OUTPUT " THRESHOLD: ";
		print OUTPUT "UHS\n" if $threshold[$j] eq "000";
		print OUTPUT "VHS\n" if $threshold[$j] eq "100";
		print OUTPUT "MHS\n" if $threshold[$j] eq "010";
		print OUTPUT "HS\n" if $threshold[$j] eq "110";
		print OUTPUT "LS\n" if $threshold[$j] eq "001";
		print OUTPUT "MLSW\n" if $threshold[$j] eq "101";
		print OUTPUT "VLS\n" if $threshold[$j] eq "011";
		print OUTPUT "ULS\n" if $threshold[$j] eq "111";

		print OUTPUT " OUTPUT: ";
		print OUTPUT "S\n" if $output[$j] eq "000";
		print OUTPUT "W\n" if $output[$j] eq "100";
		print OUTPUT "N\n" if $output[$j] eq "010";
		print OUTPUT "E\n" if $output[$j] eq "110";
		print OUTPUT "R\n" if $output[$j] eq "001";
		print OUTPUT "ClkA\n" if $output[$j] eq "101";
		print OUTPUT "ClkB\n" if $output[$j] eq "011";
		print OUTPUT "F\n" if $output[$j] eq "111";
		print OUTPUT "\n";
	    }#for

	    } #decompose

    print STDOUT "\n...My work is done here\n\n" if $verbose;

#end of script






